/**
 *	--- Program written for the PIC-18F4455 microcontroller ---
 * Uses button D0 and the CCP1 module
 * Functionality: - allow the selection of two melodies and three play speeds
 * 				  - the melody number and play speed are displayed on a 7 seg. display
 *					(not at the same time!)
 * Implementation:
 *	- The current selected melody is always played
 *  - There are two states: S0 and S1
 *		S0 : normal functionality. Using D0, one can select the play speed 
 *			 of the current melody.
 *		S1 : melody selection functionality. Using D0, one can select the 
 *			 current melody to be played.
 *  - State functionality : the dot from the 7 seg. is used to distinguish visually between
 *			 the two states, S0 and S1. In S0, the dot is off while in S1 the dot is on. 
 *		S0 : displays the speed number on the 7 seg. display. On every rising edge of D0,
 *			 (when releasing the button) the play speed of the current melody is changed. 
 *			 There are 3 speeds obtained by dividing the length of each note.
 *		S1 : displays the current melody number (with dot on!). Allows the selection of
 *			 a new melody which becomes the new melody to be played only when returning in state S0.
 *  - State transitions : by pressing D0 more than 1 second, one may toggle the states
 *		S0 : one can leave S0 (and enter S1) by pressing D0 for more than 1 second. The current 
 *			 melody keeps on playing.
 *		S1 : one can leave S1 (and enter S0) by pressing D0 for more than 1 second. The selected
 *			 melody becomes active only after leaving state S1.
 *
 * Here are the frequencies of the most common notes used in this program:
 *
 *		C4 - F = 262 Hz, T = 3.8167 mSec, T/2 = 1.9083
 *		D4 - F = 294 Hz, T = 3.4013 mSec, T/2 = 1.7006
 *		E4 - F = 330 Hz, T = 3.0303 mSec, T/2 = 1.5151
 *		F4 - F = 349 Hz, T = 2.8653 mSec, T/2 = 1.4326
 *		G4 - F = 392 Hz, T = 2.5510 mSec, T/2 = 1.2755
 *		A4 - F = 440 Hz, T = 2.2727 mSec, T/2 = 1.13635
 *		B4 - F = 494 Hz, T = 2.0246 mSec, T/2 = 1.0123
 *		C4 - F = 524 Hz, T = 1.9110 mSec, T/2 = 0.9555
 **/

#include <delays.h>
#include <pwm.h>

enum{false, true};
typedef unsigned char bool;


#include "main.h"

unsigned int  g_uNoteCounter = 0;
bool 		  g_bPlayMode = true;	//true - play mode ; false - programming mode
unsigned char g_uSpeed = 1;
unsigned int  g_uNrPressedD0 = 0;
unsigned char g_uCounterBasicDelay = 0;
unsigned char g_uMelodyNumber = 0;
unsigned char g_uTempMelodyNumber = 0;
unsigned char* g_pucCurrentMelody = g_pucMelody;
unsigned char* g_pucTempCurrentMelody = 0;
unsigned char* g_pucCurrentNoteLength = g_pucNoteLength;
unsigned char* g_pucTempCurrentNoteLength = 0;

#pragma code

void playNote( unsigned char* pucMelody, const unsigned int uNoteNr )
{
	// play note
	OpenPWM1( g_pucPeriod[ pucMelody[ uNoteNr ] ] );
	SetDCPWM1( 100 );
}

void noSound( void )
{
	// stop PWM
	ClosePWM1();
}

void changePlayMode( void )
{
	if ( g_bPlayMode == true )
	{
		g_bPlayMode = false;
	
		// display the current melody number
		PORTB  = g_puc7SegNumbers[ g_uMelodyNumber ];
		PORTB &= ~(1<<6);	// dot LED on
	
		g_pucTempCurrentMelody		= g_pucCurrentMelody;
		g_pucTempCurrentNoteLength	= g_pucCurrentNoteLength;
		g_uTempMelodyNumber	 		= g_uMelodyNumber;
	}
	else
	{
		g_bPlayMode = true;

		if ( g_pucTempCurrentMelody != g_pucCurrentMelody )
		{
			g_pucCurrentMelody 		= g_pucTempCurrentMelody;
			g_pucCurrentNoteLength	= g_pucTempCurrentNoteLength;
			g_uMelodyNumber    = g_uTempMelodyNumber;
			g_uSpeed		   = 1;
			g_uNoteCounter	   = 0;
		}

		// display the current speed number
		PORTB  = g_puc7SegNumbers[ g_uSpeed - 1 ]; 
	}
}

void onButtonReleased( void )
{
	if ( g_bPlayMode == true ) // if we are simply playing and adjusting speed
	{
		++g_uSpeed;
	
		// reset speed if needed
		if ( g_uSpeed == 4 )
		{
			g_uSpeed = 1;
		}
						
		// display current speed
		PORTB = g_puc7SegNumbers[ g_uSpeed - 1 ];
	}
	else // we can select melody
	{
		// move to next melody
		++g_uTempMelodyNumber;
	
		switch( g_uTempMelodyNumber ){
		case 0:
			g_pucTempCurrentMelody		= g_pucMelody; 
			g_pucTempCurrentNoteLength	= g_pucNoteLength;
			break;
		case 1:
			g_pucTempCurrentMelody		= g_pucMelody1;
			g_pucTempCurrentNoteLength	= g_pucNoteLength1;
			break;
		default:
			g_pucTempCurrentMelody 		= g_pucMelody;
			g_pucTempCurrentNoteLength	= g_pucNoteLength;
			g_uTempMelodyNumber			= 0;
		}
	
		// display the current melody number
		PORTB  = g_puc7SegNumbers[ g_uTempMelodyNumber ];
		PORTB &= ~(1<<6);	// dot LED on
	}
}

void main()
{
	bool bNoSound = false;
	unsigned char uNoSoundLen = INNER_DELAY;

	// select frequency - 110 : 4Mhz
	OSCCONbits.IRCF2 = 1;
	OSCCONbits.IRCF1 = 1;
	OSCCONbits.IRCF0 = 0;

	// select internal oscilator
	OSCCONbits.SCS1 = 1;
	OSCCONbits.SCS0 = 0;

	// setup timer prescaler - 1:16 (MAX) needed by the PWM module
	T2CONbits.T2CKPS1 = 1;
	T2CONbits.T2CKPS0 = 0;

	// start timer 2
	T2CONbits.TMR2ON = 1;

	// configure PORTC and PORTB as output
	TRISC = 0X00;
	TRISB = 0X00;

	// configure PORTD
	TRISDbits.TRISD0 = 1;
	TRISDbits.TRISD1 = 1;
	TRISDbits.TRISD2 = 0;
	TRISDbits.TRISD3 = 0;

	PORTDbits.RD2 = 0;
	PORTDbits.RD3 = 0;

	// display first speed
	PORTB = g_puc7SegNumbers[ g_uSpeed - 1 ];

	while( 1 )
	{
		if ( bNoSound == false )
		{
			// play note
			playNote( g_pucCurrentMelody, g_uNoteCounter );
		}
		else
		{
			noSound();
		}

		// if we detected a pressed button
		if ( PORTDbits.RD0 == 0 )
		{
			// increment the number of times that D0 has been pressed (every BASIC_DELAY)
			if ( g_uNrPressedD0 < 100 )
			{
				++g_uNrPressedD0;
			}
			// if button D0 has been pressed for more than 1 second, change mode
			else if ( g_uNrPressedD0 == 100 )// BASIC_DELAY * 100 = 1Second
			{
				changePlayMode();

				// we increment it again so that it does not enter here again
				++g_uNrPressedD0;
			}
		}
		else // when the button is released, we move into action
		{
			if ( ( g_uNrPressedD0 > 0 ) && ( g_uNrPressedD0 != 101 ) )
			{
				onButtonReleased();
			}
			g_uNrPressedD0 = 0;
		}
		
		// wait 10 millis
		Delay10KTCYx( BASIC_DELAY );

		// we count the number of 10 millis that have passed so that we know when to
		// switch to the next note
		g_uCounterBasicDelay += BASIC_DELAY;

		if ( bNoSound == false )
		{
			// check if we can move to next note
			if ( ( g_pucCurrentNoteLength[ g_uNoteCounter ] / g_uSpeed ) <= g_uCounterBasicDelay )
			{
				bNoSound	= true;
				uNoSoundLen = INNER_DELAY;

				g_uCounterBasicDelay = 0;
			}
		}
		else	// check pause between notes or melody ending
		{
			if ( ( uNoSoundLen / g_uSpeed ) <= g_uCounterBasicDelay )
			{
				if ( uNoSoundLen != MELODY_END )
				{
					++g_uNoteCounter;
		
					// check if we have reached the end of our song
					if ( g_pucCurrentMelody[ g_uNoteCounter ] == __END_NOTE__ )
					{
						g_uNoteCounter = 0;
						uNoSoundLen = MELODY_END;
					}
					else
					{
						bNoSound = false;
					}
				}
				else
				{
					bNoSound = false;
				}
				g_uCounterBasicDelay = 0;
			} 
		}
	}
}
